package affe_banane;

import java.util.ArrayList;

import suchbaum.State;


public class ABState implements State {
	
	/**
	 * Diese Klasse stellt die Zustnde des Affe-Banane-Problems dar.  
	 * @author Socher
	 *
	 */
	
	public enum Position {
		TUER, FENSTER, MITTE	
	}
	
	public enum Pos_Vertikal {
		BODEN, KISTE
	}

	
	Position hp;		// Horizontale Position des Affen
	Position pk;		// Position der Kiste
	Pos_Vertikal vp;	// Vertikale Position des Affen
	boolean banane;		// hat er die Banane?
	
	ArrayList<State> successors = new ArrayList<State>(); 	// Hier werden die Nachfolgezustnde gesammelt

	
	
	
	public ABState() {
		super();
		this.hp = Position.TUER;
		this.pk = Position.FENSTER;
		this.vp = Pos_Vertikal.BODEN;
		this.banane = false;
	}

	public ABState(Position hp, Position pk, Pos_Vertikal vp, boolean banane) {
		super();
		this.hp = hp;
		this.pk = pk;
		this.vp = vp;
		this.banane = banane;
	}
	
	/**
	 * 
	 * @return eine Kopie des Zustands
	 */
	private ABState copy(){
		return new ABState(this.hp,this.pk,this.vp,this.banane);
	}
	/* (non-Javadoc)
	 * @see affe_banane.State#isGoal()
	 */
	public boolean isGoal(){
		return banane;
	}
	/**
	 * 
	 * @param newPos eine horizontale Position 
	 * @return Folgezustand, falls die Aktion ausfhrbar ist, ansonsten null
	 */
	public ABState laufen(Position newPos){
		if (vp == Pos_Vertikal.KISTE || hp == newPos) return null;
		ABState result = this.copy();
		result.hp = newPos;
		return result; 
	}
	
	/**
	 * 
	 * @return Folgezustand, falls die Aktion ausfhrbar ist, ansonsten null
	 */
	ABState klettern(){
		if (hp != pk || vp == Pos_Vertikal.KISTE) return null;
		ABState result = this.copy();
		result.vp = Pos_Vertikal.KISTE;
		return result; 		
	}
	
	/**
	 * 
	 * @param newPos eine horizontale Position 
	 * @return Folgezustand, falls die Aktion ausfhrbar ist, ansonsten null
	 */
	public ABState schieben(Position newPos){
		if (hp != pk || vp == Pos_Vertikal.KISTE || hp == newPos) return null;
		ABState result = this.copy();
		result.hp = newPos;
		result.pk = newPos;
		return result; 		
	}
	
	/**
	 * 
	 * @return Folgezustand, falls die Aktion ausfhrbar ist, ansonsten null
	 */
	ABState holen(){
		if (vp != Pos_Vertikal.KISTE || pk != Position.MITTE || banane) return null;
		ABState result = this.copy();
		result.banane = true;
		return result; 		
	}
	
	public ArrayList<State> successors(){
		successors.clear();
		addToSucc(holen());
		addToSucc(klettern());
		addToSucc(schieben(Position.FENSTER));
		addToSucc(schieben(Position.TUER));
		addToSucc(schieben(Position.MITTE));
		addToSucc(laufen(Position.FENSTER));
		addToSucc(laufen(Position.TUER));
		addToSucc(laufen(Position.MITTE));
		return successors;
	}
	
	private void addToSucc(ABState state){
		if (state != null) successors.add(state);
	}
	
	

	
	/* (non-Javadoc)
	 * @see affe_banane.State#equals(affe_banane.ABState)
	 */
	public boolean equals(State other){
		return this.hp == ((ABState)other).hp && 
		this.pk == ((ABState)other).pk && 
		this.vp == ((ABState)other).vp && 
		this.banane == ((ABState)other).banane;
	}
	/* (non-Javadoc)
	 * @see affe_banane.State#toString()
	 */
	public String toString(){
		return "("+hp+", "+vp+", "+pk+", "+banane+")";
	}

	@Override
	public int hValue() {
		return 0;
	}

}
