package kannibalen;

import java.util.ArrayList;

import suchbaum.State;

public class MKState implements State{
	/**
	 * Diese Klasse stellt die Zustnde des Missionare-Kannibalen-Problems dar.
	 * @author Socher
	 */

	int mLinks;			// Anzahl der Missionare auf der linken Seite
	int kLinks;			// Anzahl der Kannibalen auf der linken Seite
	boolean bLinks;		// Befindet sich das Boot am linken Ufer?
	
	ArrayList<State> successors; 	// Hier werden die Nachfolgezustnde gesammelt

	
	
	public MKState() {
		super();
		mLinks = 3;
		kLinks = 3;
		bLinks = true;
		successors = new ArrayList<State>();
	}
	
	public MKState(int links, int links2, boolean boot) {
		super();
		mLinks = links;
		kLinks = links2;
		this.bLinks = boot;
		successors = new ArrayList<State>();
	}
	
	/**
	 * @return Anzahl der Missionare rechts
	 */
	private int mRechts(){
		return 3-mLinks;
	}
	
	/**
	 * @return Anzahl der Kannibalen rechts
	 */
	private int kRechts(){
		return 3-kLinks;
	}
	
	/**
	 * @return Boot rechts?
	 */
	private boolean bRechts(){
		return !bLinks;
	}
	
	/**
	 * @return Ist der Zustand sicher?
	 */
	private boolean safe(){
		return (kRechts() <= mRechts() || mRechts() == 0) && 
			(kLinks <= mLinks || mLinks == 0);
	}
	
	/**
	 * @return Ist der Zustand konsistent?
	 */
	private boolean consistent(){
		return (kRechts() >= 0 && mRechts() >= 0 && 
			kLinks >= 0 && mLinks >= 0);
	}
	
	
	/**
	 * @return Handelt es sich um einen Zielzustand?
	 */
	public boolean isGoal(){
		return mLinks == 0 && kLinks == 0;
	}
	

	/**
	 * berfahrt von links nach rechts
	 * @param anzM Anzahl M im Boot
	 * @param anzK Anzahl K im Boot
	 * @return neuer Zustand
	 */
	private MKState left2right(int anzM, int anzK){
		MKState newState  = new MKState(mLinks-anzM,kLinks-anzK,false);
		if(newState.consistent() && newState.safe() ) return newState; else return null;
	}
	
	/**
	 * berfahrt von rechts nach links
	 * @param anzM Anzahl M im Boot
	 * @param anzK Anzahl K im Boot
	 * @return neuer Zustand
	 */
	private MKState right2left(int anzM, int anzK){
		MKState newState  = new MKState(mLinks+anzM,kLinks+anzK,true);
		if(newState.consistent() && newState.safe()) return newState; else return null;
	}
	
	public String toString(){
		return "(M: "+mLinks+", K: "+kLinks+", B: "+(bLinks?"T":"F")+
			"| M: "+mRechts()+", K: "+kRechts()+", B: "+(bRechts()?"T":"F")+")";
	}

	@Override
	public boolean equals(State other) {
		return (this.bLinks == ((MKState)other).bLinks &&
				this.kLinks == ((MKState)other).kLinks &&
				this.mLinks == ((MKState)other).mLinks);
	}

	@Override
	public ArrayList<State> successors() {
		successors.clear();
		if (bLinks){
			addToSucc(left2right(1,0));		
			addToSucc(left2right(2,0));		
			addToSucc(left2right(0,1));		
			addToSucc(left2right(0,2));		
			addToSucc(left2right(1,1));	
		} else {	
			addToSucc(right2left(1,0));		
			addToSucc(right2left(0,1));		
			addToSucc(right2left(2,0));		
			addToSucc(right2left(0,2));		
			addToSucc(right2left(1,1));	
		}
		return successors;
	}
	
	private void addToSucc(MKState state){
		if (state != null) successors.add(state);
	}

	@Override
	public int hValue() {
		return 0;
	}
	

	
	
}
