package schiebepuzzle;

import java.util.ArrayList;
import java.util.Arrays;

import suchbaum.State;

public class SPState implements State {
	
	int zeile;		// Zeile des leeren Feldes
	int spalte;		// Spalte des leeren Feldes
	int[][] feld;	// Spielfeld
	
	ArrayList<State> successors; 						// Hier werden die Nachfolgezustnde gesammelt
	
	static final int[][] START = {{7,2,3},{1,0,6},{5,4,8}};	
	static final int[][] GOAL = {{1,2,3},{8,0,4},{7,6,5}};	// siehe Beispiel im Buch (S. 32)
	static final int[][] GOALPOSITIONS = 
		{{1,1},{0,0},{0,1},{0,2},{1,2},{2,2},{2,1},{2,0},{1,0} };	// ACHTUNG: Wird GOAL gendert, muss auch GOALPOSITIONS gendert werden!
	
	
	public enum Valuation {								// Die beiden Heuristiken von S. 32, Beispiel 2.1
		H1, H2	
	}
	
	static final Valuation valuation = Valuation.H2;			// Kann hier gendert werden


	public SPState() throws Exception {
		super();
		this.feld = START;
		boolean found = false;
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 3; j++)
				if (feld[i][j] == 0){
					zeile = i;
					spalte = j;
					found = true;
				}
		if (!found) throw new Exception("Keine Null im Startfeld vorhanden!");
		successors = new ArrayList<State>();
	}

	public SPState(int zeile, int spalte, int[][] feld) {
		super();
		this.zeile = zeile;
		this.spalte = spalte;
		this.feld = feld;
		successors = new ArrayList<State>();
	}
	
	public static SPState ziel(){
		return new SPState(1,1,GOAL);
	}
	
	private int[][] copyField(){
		int feld2[][] = new int[3][3];
		for (int i = 0;i < 3;i++)
			for (int j = 0;j < 3;j++)
				feld2[i][j] = feld[i][j];
		return feld2;
	}
	
	private SPState east(){
		if (spalte == 2) return null;
		int feld2[][] = copyField();
		feld2[zeile][spalte] = feld2[zeile][spalte+1];
		feld2[zeile][spalte+1] = 0;
		return new SPState(zeile,spalte+1,feld2);
	}

	private SPState north(){
		if (zeile == 0) return null;
		int feld2[][] = copyField();
		feld2[zeile][spalte] = feld2[zeile-1][spalte];
		feld2[zeile-1][spalte] = 0;
		return new SPState(zeile-1,spalte,feld2);
	}

	private SPState south(){
		if (zeile == 2) return null;
		int feld2[][] = copyField();
		feld2[zeile][spalte] = feld2[zeile+1][spalte];
		feld2[zeile+1][spalte] = 0;
		return new SPState(zeile+1,spalte,feld2);
	}

	private SPState west(){
		if (spalte == 0) return null;
		int feld2[][] = copyField();
		feld2[zeile][spalte] = feld2[zeile][spalte-1];
		feld2[zeile][spalte-1] = 0;
		return new SPState(zeile,spalte-1,feld2);
	}

	@Override
	public boolean equals(State other) {
		return (this.spalte == ((SPState)other).spalte &&
				this.zeile == ((SPState)other).zeile &&
				Arrays.deepEquals(this.feld, ((SPState)other).feld));
	}
	

	@Override
	public boolean isGoal() {
		return Arrays.deepEquals(feld, GOAL);
	}

	@Override
	public ArrayList<State> successors() {
		successors.clear();
		addToSucc(north());
		addToSucc(south());
		addToSucc(west());
		addToSucc(east());
		return successors;
	}
	
	private void addToSucc(SPState state){
		if (state != null) successors.add(state);
	}

	
	public String toString(){
		String result = "";
		for (int i = 0;i < 3;i++){
			for (int j = 0;j < 3;j++)
				result = result + (feld[i][j]==0?"X":feld[i][j])+"|";
			result = result + "\n------\n";
		}
		result = result + "Wert: "+this.hValue()+"\n";
		return result;
	}

	@Override
	public int hValue() {
		if (valuation == Valuation.H1) return value1();
		else return value2();
	}
	
	/**
	 * realisiert Heuristik h1 (S. 32, Beispiel 2.1)
	 * @return der heuristische Wert des Zustands
	 */
	private int value1(){
		int diff = 0;
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 3; j++)
				diff = diff + ( this.feld[i][j] == GOAL[i][j] ? 0 : 1);
		return diff;
	}

	/**
	 * realisiert Heuristik h2 (S. 32, Beispiel 2.1)
	 * @return der heuristische Wert des Zustands
	 */
	private int value2(){
		int diff = 0;
		int val;
		for (int i = 0; i < 3; i++)
			for (int j = 0; j < 3; j++){
				val = this.feld[i][j];
				if (val != 0)
					diff = diff + Math.abs(GOALPOSITIONS[val][0]-i)+Math.abs(GOALPOSITIONS[val][1]-j);
			}
		return diff;
	}

}
