package suchbaum;

import java.util.ArrayList;


public class SearchNode implements Comparable<SearchNode>{

	/**
	 * Diese Klasse stellt Knoten im Suchbaum dar
	 * @author Socher
	 */
	protected State aktuell;
	protected SearchNode previous;
	protected int depth;

	public SearchNode() {
		super();
	}

	public SearchNode(State aktuell) {
		super();
		this.aktuell = aktuell;
		this.previous = null;
		this.depth = 0;		// Wurzelknoten
	}

	public SearchNode(State aktuell, SearchNode previous) {
		super();
		this.aktuell = aktuell;
		this.previous = previous;
		this.depth = previous.depth + 1;
	}


	/**
	 * @return true, wenn der Knoten ein Zielknoten ist
	 */
	public boolean isGoal() {
		return this.aktuell.isGoal();
	}
	
	/**
	 * @return eine Liste der Nachfolgeknoten des aktuellen Knotens
	 */
	public ArrayList<SearchNode> expand(){
		ArrayList<SearchNode> result = new ArrayList<SearchNode>();
		for(State st : aktuell.successors())	
			if (!this.containsState(st)) result.add(new SearchNode(st,this));
		return result;
	}

/**
 * 
 * @param maxDepth maximale Tiefe, bis zu der die Nachfolgeknoten noch aufgesammelt werden,
 * wird bentigt fr schrittweise Vertiefung
 * @return eine Liste der Nachfolgeknoten des aktuellen Knotens
 */
	public ArrayList<SearchNode> expandUpToN(int maxDepth){
		ArrayList<SearchNode> result = new ArrayList<SearchNode>();
		for(State st : aktuell.successors())	
			if (!this.containsState(st) && this.depth < maxDepth) result.add(new SearchNode(st,this));
		return result;
	}


	public String toString() {
   		return  (previous!= null ? previous+ "\n" : "")  + aktuell ;
	}


	/**
	 * @param state Ein Zustand
	 * @return true, wenn der Zustand schon einmal durchlaufen wurde
	 */
	private boolean containsState(State state) {
		return aktuell.equals(state) || (previous != null && previous.containsState(state));
	}

	/**
	 * Fr die Bestensuche
	 * Liefert den Vergleich zweier Knoten nach ihrem heuristischem Wert
	 */
	
	public int compareTo(SearchNode arg0) {
		return (this.aktuell.hValue() > arg0.aktuell.hValue() ? 1 : -1);
	}
	
	/**
	 * wird gebraucht fr die bidirektionale Suche
	 * @param other ein anderer Knoten (Rckwrtssuche)
	 * @return true, wenn der aktuelle Knoten und der andere Knoten den selben aktuellen Zustand haben
	 * (Vorwrtssuche und Rckwrtssuche treffen sich in der Mitte)
	 */
	public boolean meets(SearchNode other){
		return (this.aktuell.equals(other.aktuell));
	}

	/**
	 * wird gebraucht fr die bidirektionale Suche
	 * @param otherNode ein anderer Knoten (Rckwrtssuche)
	 * @return die beiden werden aneinandergehngt. Dabei wird der zweite Knoten "umgedreht"
	 * d.h. von Blatt zur Wurzel drangehngt. 
	 */
	public SearchNode join(SearchNode otherNode) {
		SearchNode result = new SearchNode(this.aktuell,this.previous);
		while (otherNode.previous != null){
			result = new SearchNode(otherNode.aktuell,result);
			otherNode = otherNode.previous;
		}
		result = new SearchNode(otherNode.aktuell,result);
		return result;
	}

	

}