package suchverfahren;

import java.util.ArrayList;
import java.util.PriorityQueue;

import suchbaum.SearchNode;



public class Suche {
	/**
	 * Diese Klasse enthlt die verschiedenen Suchverfahren
	 * @author Socher
	 */
	
	
	static final int MAX = 10000;			// vorsichtshalber...
	
	static int counter;						// zhlt die Expansionsvorgnge
	
	/**
	 * Realisiert die Breitensuche (Breadth-First-Search)
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	
	/**
	 * dynamische Initialisierung
	 */
	private static void init(){
		counter = 0;
	}
	
	public static int getCounter(){
		return counter;
	}
	
	/**
	 * Realisiert die Breitensuche (Breadth-First-Search)
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	
	public static SearchNode bfs(SearchNode start){
		init();
		ArrayList<SearchNode> agenda = new ArrayList<SearchNode>();
		agenda.add(start);
		while (!agenda.isEmpty() && counter < MAX ){
			counter++;
			SearchNode aktuell = agenda.get(0);
//			System.out.println(aktuell);
			agenda.remove(0);
			if (aktuell.isGoal()) return aktuell;
			for (SearchNode newSearchNode : aktuell.expand()) agenda.add(newSearchNode);
		}
		return null;
	}
	
	/**
	 * Realisiert die Tiefensuche (Depth-First-Search)
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	
	public static SearchNode dfs(SearchNode start){
		init();
		ArrayList<SearchNode> agenda = new ArrayList<SearchNode>();
		agenda.add(start);
		while (!agenda.isEmpty() && counter < MAX ){
			counter++;
			SearchNode aktuell = agenda.get(0);
//			System.out.println(aktuell);
			agenda.remove(0);
			if (aktuell.isGoal()) return aktuell;
			for (SearchNode newSearchNode : aktuell.expand()) agenda.add(0, newSearchNode);
		}
		return null;
	}
	
	/**
	 * Realisiert die schrittweise Vertiefung (Iterative Deepening))
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	public static SearchNode id(SearchNode start){
		init();
		SearchNode result = null;
		int maxDepth = 0;
		while (result == null && maxDepth < MAX){
			maxDepth++;
			counter++;
			result = idPhase(maxDepth, start);
		}
		return result;
	}
	
	private static SearchNode idPhase(int n, SearchNode start){
		ArrayList<SearchNode> agenda = new ArrayList<SearchNode>();
		agenda.add(start);
		while (!agenda.isEmpty()){
			SearchNode aktuell = agenda.get(0);
//			System.out.println(aktuell);
			agenda.remove(0);
			if (aktuell.isGoal()) return aktuell;
			for (SearchNode newSearchNode : aktuell.expandUpToN(n)) agenda.add(0, newSearchNode);
		}
		return null;
		
	}
	
	/**
	 * realisiert die Bestensuche (Best-First Search)
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	public static SearchNode xfs(SearchNode start){
		init();
		PriorityQueue<SearchNode> agenda = new PriorityQueue<SearchNode>();
		agenda.add(start);
		while (!agenda.isEmpty() && counter < MAX ){
			counter++;
			SearchNode aktuell = agenda.poll();
//			System.out.println(aktuell);
			if (aktuell.isGoal()) return aktuell;
			for (SearchNode newSearchNode : aktuell.expand()) agenda.add(newSearchNode);
		}
		return null;
	}
	
	/**
	 * Realisiert die doppelseitige Suche (Bidirectional-Search)
	 * @param start der Startknoten des Problems
	 * @return Einen Zielknoten, falls einer gefunden wurde, ansonsten null
	 */
	
	public static SearchNode bds(SearchNode start, SearchNode ziel){
		init();
		ArrayList<SearchNode> agendaForward = new ArrayList<SearchNode>();
		ArrayList<SearchNode> agendaBackward = new ArrayList<SearchNode>();
		agendaForward.add(start);
		agendaBackward.add(ziel);
			System.out.println("Start: \n");
		while (!agendaForward.isEmpty() && !agendaBackward.isEmpty() && counter < MAX ){
			counter++;
			SearchNode aktuellF = agendaForward.get(0);
			SearchNode aktuellB = agendaBackward.get(0);
//			System.out.println("Forward: \n"+aktuellF);
//			System.out.println("Backward: \n"+aktuellB);
			agendaForward.remove(0);
			agendaBackward.remove(0);
			if (aktuellF.isGoal()) return aktuellF;
			for (SearchNode nF : agendaForward)
				if (nF.meets(aktuellB)) return nF.join(aktuellB);
			for (SearchNode newSearchNode : aktuellF.expand()) agendaForward.add(newSearchNode);
			for (SearchNode newSearchNode : aktuellB.expand()) agendaBackward.add(newSearchNode);
		}
		return null;
	}
	

		
	
}
